/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author DBogdan (dragos.bogdan@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 678
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include <xc.h>
#include "Console.h"
#include "AD7193.h"

/******************************************************************************/
/*********************** Device configuration words ***************************/
/******************************************************************************/
#pragma config POSCMOD  = XT        /*!< XT oscillator mode selected */
#pragma config FNOSC    = PRIPLL    /*!< Primary Oscillator with PLL Module*/
#pragma config FPLLMUL  = MUL_20    /*!< 20x multiplier */
#pragma config FPLLIDIV = DIV_2     /*!< 2x divider */
#pragma config FPLLODIV = DIV_1     /*!< PLL output divided by 1 */
#pragma config FPBDIV   = DIV_1     /*!< PBCLK is SYSCLK divided by 1 */
#pragma config FWDTEN   = OFF       /*!< The WDT is not enabled */

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/
/*! List of available commands */
const char* commadsList[] =
    {"help?",
     "reset=",
     "mode?",
     "mode=",
     "pseudo?",
     "pseudo=",
     "channel?",
     "channel=",
     "data?",
     "voltage?",
     "temperature?"};
const char* commandsDescription[] =
    {"  -  Displays all available commands.",
     "  -  Resets the AD7193.",
     "  -  Displays the selected operating mode.",
     "  -  Selects the AD7193's operating mode. Accepted values: 0 - 3.",
     "  -  Displays the Pseudo Bit value (AD7193_REG_CONF).",
     "  -  Sets the Pseudo Bit Value (AD7193_REG_CONF). Accepted values: 0, 1.",
     "  -  Displays the enabled channel.",
     "  -  Enables one channel on the AD7193. Accepted values: 0 ? 7.",
     "  -  Displays the value of the data register (AD7193_REG_DATA).",
     "  -  Displays the voltage applied to enabled channel.",
     "  -  Displays the temperature."};

/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned char   commandsNumber = (sizeof(commadsList) / sizeof(const char*));
unsigned char   receivedCommand[20];
unsigned char   invalidCommand = 0;
unsigned char   commandType = 0;
unsigned char   command = 0;
unsigned char   displayCommand = 0;
double          commandParam = 0;

/***************************************************************************//**
 * @brief Converts a float value to a character array with 3 digits of accuracy.
 *
 * @param *buf - returns the converterd value
 * @param val - value to be converted
 *
 * @return None.
*******************************************************************************/
void FloatToString(char * buf, double val)
{
    long  intPart  = 0;
    short fracPart = 0;
    char  charPos  = 0;
    char  localBuf[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    char  i = sizeof(localBuf) - 1;

    intPart = (long)val;
    fracPart = (short)((val - intPart) * 1000 + 0.5);
    while(i > sizeof(localBuf) - 4)
    {
        localBuf[i] = (fracPart % 10) + 0x30;
        fracPart /= 10;
        i--;
    }
    localBuf[i] = '.';
    if(intPart == 0)
    {
        i --;
        localBuf[i] = '0';
    }
    while(intPart)
    {
        i --;
        localBuf[i] =(intPart % 10)  + 0x30;
        intPart /= 10;
    }
    for(charPos = i; charPos < sizeof(localBuf); charPos ++)
    {
        *buf = localBuf[charPos];
        buf ++;
    }
}

/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
void main(void)
{
    /*! Variables holding information about the device */
    unsigned char channel = 0;      /*!< Enabled channel */
    unsigned char pseudoBit = 0;    /*!< Pseudo Bit value */

    /*! Temporary variables */
    unsigned char   dataString[10]  = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    unsigned short  temperatureValue = 0;
    unsigned long   regValue = 0;
    unsigned long   data = 0;
    double          voltage = 0;

    /*! Initialize the UART communication peripheral. */
    UART_Init(9600);
    /*! Initialize the AD7193 device. */
    if(AD7193_Init())
    {
        CONSOLE_WriteString("AD7193 OK");
        UART_Write(0x0D);
    }
    else
    {
        CONSOLE_WriteString("AD7193 Error");
        UART_Write(0x0D);
    }
    /*! Resets the device. */
    AD7193_Reset();
    /*! Select Channel 0 */
    channel = 0;
    AD7193_ChannelSelect(AD7193_CH_0);
    /*! Calibrates channel 0. */
    AD7193_Calibrate(AD7193_MODE_CAL_INT_ZERO, AD7193_CH_0);
    /*! Selects unipolar operation and ADC's input range to +-2.5V. */
    AD7193_RangeSetup(0, AD7193_CONF_GAIN_1);
    while(1)
    {
        CONSOLE_GetCommand(receivedCommand);
        invalidCommand = 0;
        for(command = 0; command < commandsNumber; command++)
        {
            commandType = CONSOLE_CheckCommands(receivedCommand,
                                                commadsList[command],
                                                (double*)&commandParam);
            if(commandType == 0)
            {
                invalidCommand++;
            }
            if((command == 0) && (commandType != 0))    // help?
            {
                CONSOLE_WriteString("Available commands:");
                UART_Write(0x0D);
                for(displayCommand = 0; displayCommand < commandsNumber; displayCommand++)
                {
                    CONSOLE_WriteString(commadsList[displayCommand]);
                    CONSOLE_WriteString(commandsDescription[displayCommand]);
                    UART_Write(0x0D);
                }
            }
            if((command == 1) && (commandType != 0))    // reset=
            {
                CONSOLE_WriteString(commadsList[command]);
                if(commandParam == 1)
                {
                    AD7193_Reset();
                    CONSOLE_WriteString("1");
                }
                else
                {
                    CONSOLE_WriteString("0");
                }
                UART_Write(0x0D);
            }
            if((command == 2) && (commandType != 0))    // mode?
            {
                CONSOLE_WriteString(commadsList[command + 1]);
                regValue = AD7193_GetRegisterValue(AD7193_REG_MODE, 3, 1) & AD7193_MODE_SEL(0x7);
                regValue >>= 21;
                itoa(dataString, regValue, 10);
                CONSOLE_WriteString(dataString);
                UART_Write(0x0D);
            }
            if((command == 3) && (commandType != 0))    // mode=
            {
                CONSOLE_WriteString(commadsList[command]);
                switch((unsigned char)commandParam)
                {
                    case 1:
                        AD7193_SetMode(1);
                        CONSOLE_WriteString("1");
                        break;
                    case 2:
                        AD7193_SetMode(2);
                        CONSOLE_WriteString("2");
                        break;
                    case 3:
                        AD7193_SetMode(3);
                        CONSOLE_WriteString("3");
                        break;
                    default:
                        AD7193_SetMode(0);
                        CONSOLE_WriteString("0");
                        break;
                }
                UART_Write(0x0D);
            }
            if((command == 4) && (commandType != 0))    // pseudo?
            {
                CONSOLE_WriteString(commadsList[command + 1]);
                UART_Write(pseudoBit + 0x30);
                UART_Write(0x0D);
            }
            if((command == 5) && (commandType != 0))    // pseudo=
            {
                CONSOLE_WriteString(commadsList[command]);
                switch((unsigned char)commandParam)
                {
                    case 1:
                        pseudoBit = 1;
                        regValue = AD7193_GetRegisterValue(AD7193_REG_CONF, 3, 1);
                        regValue |= AD7193_CONF_PSEUDO;
                        AD7193_SetRegisterValue(AD7193_REG_CONF, regValue, 3, 1);
                        CONSOLE_WriteString("1");
                        break;
                    default:
                        pseudoBit = 0;
                        regValue = AD7193_GetRegisterValue(AD7193_REG_CONF, 3, 1);
                        regValue &= ~ AD7193_CONF_PSEUDO;
                        AD7193_SetRegisterValue(AD7193_REG_CONF, regValue, 3, 1);
                        CONSOLE_WriteString("0");
                        break;
                }
                UART_Write(0x0D);
            }
            if((command == 6) && (commandType != 0))    // channel?
            {
                CONSOLE_WriteString(commadsList[command + 1]);
                UART_Write(channel + 0x30);
                UART_Write(0x0D);
            }
            if((command == 7) && (commandType != 0))    // channel=
            {
                CONSOLE_WriteString(commadsList[command]);
                if(commandParam >=0 && commandParam <= 7)
                {
                    channel = (unsigned char)commandParam;
                }
                else
                {
                    if(commandParam < 0)
                    {
                        channel = 0;
                    }
                    else
                    {
                        channel = 7;
                    }
                }
                AD7193_ChannelSelect(channel);
                UART_Write(channel + 0x30);
                UART_Write(0x0D);
            }
            if((command == 8) && (commandType != 0))    // data?
            {
                data = AD7193_SingleConversion();
                itoa(dataString, data, 10);
                CONSOLE_WriteString("data=");
                CONSOLE_WriteString(dataString);
                UART_Write(0x0D);
            }
            if((command == 9) && (commandType != 0))    // voltage?
            {
                data = AD7193_SingleConversion();
                voltage = (double)(2.5 / 16777215) * data;
                FloatToString(dataString, voltage);
                CONSOLE_WriteString("voltage=");
                CONSOLE_WriteString(dataString);
                UART_Write(0x0D);
            }
            if((command == 10) && (commandType != 0))    // temperature?
            {
                temperatureValue = AD7193_TemperatureRead();
                itoa(dataString, temperatureValue, 10);
                CONSOLE_WriteString("temperature=");
                CONSOLE_WriteString(dataString);
                UART_Write(0x0D);
            }
        }
        if(invalidCommand == commandsNumber)
        {
            CONSOLE_WriteString("Invalid command");
            UART_Write(0x0D);
        }
    }
}
